package com.github.gkutiel.filter;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;


class FilterChainBuilder {
	private final Injector injector;

	FilterChainBuilder(final Injector injector) {
		this.injector = injector;
	}

	private LinkedList<Filter> build(final Filter outer, final List<Class<? extends Filter>> classes) {
		final LinkedList<Filter> filters;
		if (classes.isEmpty()) filters = new LinkedList<>();
		else filters = build(newFilter(outer, classes.remove(0)), classes);
		filters.push(outer);
		return filters;
	}

	private List<Class<? extends Filter>> filters(final Class<?> inner) {
		if (inner == null || !Filter.class.isAssignableFrom(inner)) return new ArrayList<>();
		final List<Class<? extends Filter>> filters = filters(inner.getEnclosingClass());
		filters.add(inner.asSubclass(Filter.class));
		return filters;
	}

	private Filter newFilter(final Class<? extends Filter> filter) {
		try {
			final Constructor<? extends Filter> ctor = filter.getDeclaredConstructor();
			ctor.setAccessible(true);
			return injector.inject(ctor.newInstance());
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	private Filter newFilter(final Filter outer, final Class<? extends Filter> filter) {
		try {
			final Constructor<? extends Filter> ctor = filter.getDeclaredConstructor(outer.getClass());
			ctor.setAccessible(true);
			return injector.inject(ctor.newInstance(outer));
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	LinkedList<Filter> build(final Class<? extends Filter> inner) {
		final List<Class<? extends Filter>> filters = filters(inner);
		return build(newFilter(filters.remove(0)), filters);
	}
}
