package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.DefaultParsers.boolParser;
import static com.github.gkutiel.filter.DefaultParsers.stringParser;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.servlet.ServletHandler;
import org.eclipse.jetty.servlet.ServletHolder;

public class FilterServer implements Parser {
	class InjectorBuilder {
		Injector build(final HttpServletRequest req, final HttpServletResponse res) {
			final String contentType = req.getContentType();
			if (contentType != null && contentType.startsWith("multipart/form-data;")) return new MultipartInjector(parsers, req, res);
			return new UrlEncodedInjector(parsers, req, res);
		}
	}

	private final HandlerList handlers = new HandlerList();
	private final InjectorBuilder injectorBuilder = new InjectorBuilder();
	private final Map<Class<?>, ParamParser<?>> parsers = new HashMap<>();

	private final Server server;

	private final ServletHandler servletHandler = new ServletHandler();

	public FilterServer(final int port) {
		server = new Server(port);
		handlers.addHandler(new ResourceHandler());
		handlers.addHandler(servletHandler);
		server.setHandler(handlers);

		parsers.put(String.class, stringParser);
		parsers.put(boolean.class, boolParser);
		parsers.put(Boolean.class, boolParser);
	}

	@Override
	public <T extends Api> Binder bind(final String path, final Class<? extends Remote<T>> ws, final Class<T> remote) {
		servletHandler.addServletWithMapping(new ServletHolder(new WebSocketHandler<>(ws, remote, injectorBuilder)), path);
		return this;
	}

	@Override
	public Mapper map(final String path, final Class<? extends Filter> filter) {
		servletHandler.addServletWithMapping(new ServletHolder(new FilterHandler(filter, injectorBuilder) {}), path);
		return this;
	}

	@Override
	public <T> Parser parse(final Class<T> c, final ParamParser<T> parser) {
		parsers.put(c, parser);
		return this;
	}

	public void start() {
		try {
			server.start();
			server.join();
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

}
