package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.Util.getCookie;
import static java.lang.reflect.AccessibleObject.setAccessible;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.github.gkutiel.filter.Filter.Cookie;
import com.github.gkutiel.filter.Filter.Param;
import com.github.gkutiel.filter.Filter.SignedValue;
import com.github.gkutiel.filter.Val.StringVisitor;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

public abstract class Injector {
	private static class GsonParser implements ParamParser<Object> {

		private final Class<?> type;

		public GsonParser(final Class<?> type) {
			this.type = type;
		}

		@Override
		public Object parse(final Val val) {
			return val.accept(new StringVisitor<Object>() {

				@Override
				public Object visit(final String val) {
					return GSON.fromJson(val, type);
				}
			});
		}
	}

	protected static final Gson GSON = new Gson();

	private static List<Field> fields(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Field> fields = fields(c.getSuperclass());
		fields.addAll(Arrays.asList(c.getDeclaredFields()));
		return fields;
	}

	private final Map<Class<?>, ParamParser<?>> parsers;
	protected final HttpServletRequest req;
	protected final HttpServletResponse res;

	protected Injector(final Map<Class<?>, ParamParser<?>> parsers, final HttpServletRequest req, final HttpServletResponse res) {
		this.parsers = parsers;
		this.req = req;
		this.res = res;
	}

	private void inject(final Filter filter, final Field field) throws IllegalAccessException {
		if (field.getType().equals(HttpServletRequest.class)) field.set(filter, req);
		else if (field.getType().equals(HttpServletResponse.class)) field.set(filter, res);
		else if (field.isAnnotationPresent(Param.class)) injectVal(filter, field);
		else if (field.isAnnotationPresent(Cookie.class)) injectCookie(filter, field);
	}

	private <T> void injectCookie(final T obj, final Field field) throws JsonSyntaxException, IllegalArgumentException, IllegalAccessException {
		final String val = getCookie(req, field.getName());
		if (val == null) return;
		final SignedValue signedValue = GSON.fromJson(val, SignedValue.class);
		if (!signedValue.isValid()) return;
		field.set(obj, GSON.fromJson(signedValue.val(), field.getType()));
	}

	private void injectVal(final Filter filter, final Field field) throws JsonSyntaxException, IllegalArgumentException, IllegalAccessException {
		final String name = field.getName();
		final Class<?> type = field.getType();
		final Val val = getVal(name);
		final ParamParser<?> parser = parsers.get(type);

		final Object obj;
		if (parser != null) obj = parser.parse(val);
		else obj = new GsonParser(type).parse(val);

		if (obj != null) field.set(filter, obj);
	}

	protected abstract Val getVal(String name);

	Filter inject(final Filter filter) {
		try {
			final List<Field> fields = fields(filter.getClass());
			setAccessible(fields.toArray(new Field[fields.size()]), true);

			for (final Field field : fields)
				inject(filter, field);

			return filter;
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}
}