package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.RemoteProxy.newRemote;

import java.util.LinkedList;

import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.UpgradeRequest;
import org.eclipse.jetty.websocket.api.UpgradeResponse;
import org.eclipse.jetty.websocket.api.WebSocketAdapter;
import org.eclipse.jetty.websocket.servlet.WebSocketCreator;
import org.eclipse.jetty.websocket.servlet.WebSocketServlet;
import org.eclipse.jetty.websocket.servlet.WebSocketServletFactory;

import com.github.gkutiel.filter.Filter.RedirectException;
import com.github.gkutiel.filter.FilterServer.InjectorBuilder;

class WebSocketHandler<T extends Api> extends WebSocketServlet {

	private final Class<? extends Remote<T>> wsClass;
	private final Class<T> remoteInterface;
	private final InjectorBuilder injectorBuilder;

	WebSocketHandler(final Class<? extends Remote<T>> ws, final Class<T> remoteInterface, final InjectorBuilder injectorBuilder) {
		this.wsClass = ws;
		this.remoteInterface = remoteInterface;
		this.injectorBuilder = injectorBuilder;
	}

	@Override
	public void configure(final WebSocketServletFactory factory) {
		factory.setCreator(new WebSocketCreator() {

			@Override
			@SuppressWarnings({ "unused", "unchecked" })
			public Object createWebSocket(final UpgradeRequest req, final UpgradeResponse res) {
				try {
					final Injector injector = injectorBuilder.build(new UpgradeRequestWrapper(req), new UpgradeResponseWrapper(res));
					final LinkedList<Filter> filters = new FilterChainBuilder(injector).build(wsClass);
					final Remote<T> ws = (Remote<T>) filters.getLast();

					return new WebSocketAdapter() {
						@Override
						public void onWebSocketClose(final int statusCode, final String reason) {
							ws.bye();
						}

						@Override
						public void onWebSocketConnect(final Session session) {
							ws.hi(newRemote(session, remoteInterface));
						}
					};
				} catch (final RedirectException e) {
					e.printStackTrace();
					return null;
				}
			}
		});
	}
}
